@@ -22,14 +22,15 @@ from six.moves import cPickle as pickle
 from docutils import nodes
 from docutils.io import DocTreeInput, StringOutput
 from docutils.core import Publisher
-from docutils.utils import new_document
+from docutils.utils import new_document, relative_path
 from docutils.frontend import OptionParser
 from docutils.readers.doctree import Reader as DoctreeReader
 
 from sphinx import package_dir, __display_version__
-from sphinx.util import jsonimpl, copy_static_entry
+from sphinx.util import jsonimpl, copy_static_entry, copy_extra_entry
+from sphinx.util.i18n import format_date
 from sphinx.util.osutil import SEP, os_path, relative_uri, ensuredir, \
-    movefile, ustrftime, copyfile
+    movefile, copyfile
 from sphinx.util.nodes import inline_all_toctrees
 from sphinx.util.matching import patmatch, compile_matchers
 from sphinx.locale import _
@@ -291,7 +292,9 @@ class StandaloneHTMLBuilder(Builder):
         # typically doesn't include the time of day
         lufmt = self.config.html_last_updated_fmt
         if lufmt is not None:
-            self.last_updated = ustrftime(lufmt or _('%b %d, %Y'))
+            self.last_updated = format_date(lufmt or _('%b %d, %Y'),
+                                            language=self.config.language,
+                                            warn=self.warn)
         else:
             self.last_updated = None
 
@@ -408,6 +411,9 @@ class StandaloneHTMLBuilder(Builder):
         # metadata for the document
         meta = self.env.metadata.get(docname)
 
+        # Suffix for the document
+        source_suffix = '.' + self.env.doc2path(docname).split('.')[-1]
+
         # local TOC and global TOC tree
         self_toc = self.env.get_toc_for(docname, self)
         toc = self.render_partial(self_toc)['fragment']
@@ -425,6 +431,7 @@ class StandaloneHTMLBuilder(Builder):
             toc = toc,
             # only display a TOC if there's more than one item to show
             display_toc = (self.env.toc_num_entries[docname] > 1),
+            page_source_suffix = source_suffix,
         )
 
     def write_doc(self, docname, doctree):
@@ -508,7 +515,7 @@ class StandaloneHTMLBuilder(Builder):
         indexcounts = []
         for _k, entries in genindex:
             indexcounts.append(sum(1 + len(subitems)
-                                   for _, (_, subitems) in entries))
+                                   for _, (_, subitems, _) in entries))
 
         genindexcontext = dict(
             genindexentries = genindex,
@@ -555,12 +562,15 @@ class StandaloneHTMLBuilder(Builder):
                               (path.join(self.srcdir, src), err))
 
     def copy_download_files(self):
+        def to_relpath(f):
+            return relative_path(self.srcdir, f)
         # copy downloadable files
         if self.env.dlfiles:
             ensuredir(path.join(self.outdir, '_downloads'))
             for src in self.app.status_iterator(self.env.dlfiles,
                                                 'copying downloadable files... ',
-                                                brown, len(self.env.dlfiles)):
+                                                brown, len(self.env.dlfiles),
+                                                stringify_func=to_relpath):
                 dest = self.env.dlfiles[src][1]
                 try:
                     copyfile(path.join(self.srcdir, src),
@@ -637,11 +647,12 @@ class StandaloneHTMLBuilder(Builder):
         self.info(bold('copying extra files... '), nonl=True)
         extraentries = [path.join(self.confdir, epath)
                         for epath in self.config.html_extra_path]
+        matchers = compile_matchers(self.config.exclude_patterns)
         for entry in extraentries:
             if not path.exists(entry):
                 self.warn('html_extra_path entry %r does not exist' % entry)
                 continue
-            copy_static_entry(entry, self.outdir, self)
+            copy_extra_entry(entry, self.outdir, matchers)
         self.info('done')
 
     def write_buildinfo(self):
@@ -830,13 +841,15 @@ class StandaloneHTMLBuilder(Builder):
                      u'# The remainder of this file is compressed using zlib.\n'
                      % (self.config.project, self.config.version)).encode('utf-8'))
             compressor = zlib.compressobj(9)
-            for domainname, domain in iteritems(self.env.domains):
+            for domainname, domain in sorted(self.env.domains.items()):
                 for name, dispname, type, docname, anchor, prio in \
                         sorted(domain.get_objects()):
                     if anchor.endswith(name):
                         # this can shorten the inventory by as much as 25%
                         anchor = anchor[:-len(name)] + '$'
-                    uri = self.get_target_uri(docname) + '#' + anchor
+                    uri = self.get_target_uri(docname)
+                    if anchor:
+                        uri += '#' + anchor
                     if dispname == name:
                         dispname = u'-'
                     f.write(compressor.compress(
@@ -935,6 +948,13 @@ class SingleFileHTMLBuilder(StandaloneHTMLBuilder):
             if hashindex >= 0:
                 refnode['refuri'] = fname + refuri[hashindex:]
 
+    def _get_local_toctree(self, docname, collapse=True, **kwds):
+        if 'includehidden' not in kwds:
+            kwds['includehidden'] = False
+        toctree = self.env.get_toctree_for(docname, self, collapse, **kwds)
+        self.fix_refuris(toctree)
+        return self.render_partial(toctree)['fragment']
+
     def assemble_doctree(self):
         master = self.config.master_doc
         tree = self.env.get_doctree(master)
